"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toSubscriber = exports.isObservable = exports.inferType = exports.inferPath = void 0;
var rxjs_1 = require("rxjs");
function inferPath(observable) {
    var source = observable.source;
    if (source) {
        return inferPath(source) + "/" + inferType(observable);
    }
    return "/" + inferType(observable);
}
exports.inferPath = inferPath;
function inferType(observable) {
    var operator = observable.operator;
    var prototype = Object.getPrototypeOf(operator ? operator : observable);
    if (prototype.constructor && prototype.constructor.name) {
        var name_1 = prototype.constructor.name;
        name_1 = "" + name_1.charAt(0).toLowerCase() + name_1.substring(1);
        return name_1.replace(/^(\w+)(Observable|Operator)$/, function (match, p) { return p; });
    }
    return "unknown";
}
exports.inferType = inferType;
function isObservable(arg) {
    return arg && arg.subscribe;
}
exports.isObservable = isObservable;
var empty = {
    closed: true,
    error: function (error) {
        throw error;
    },
    next: function (value) { },
    complete: function () { },
};
var SubscriberSymbol = Symbol.for("rxSubscriber");
var SafeSubscriberCtor;
var observable = new rxjs_1.Observable(function (subscriber) {
    SafeSubscriberCtor = Object.getPrototypeOf(subscriber).constructor;
});
observable.subscribe(function () { }).unsubscribe();
function toSubscriber(observerOrNext, error, complete) {
    if (observerOrNext instanceof rxjs_1.Subscriber) {
        return observerOrNext;
    }
    var next;
    if (typeof observerOrNext === "function") {
        next = observerOrNext;
    }
    else if (observerOrNext) {
        (complete = observerOrNext.complete, error = observerOrNext.error, next = observerOrNext.next);
        next = next ? function (value) { return observerOrNext.next(value); } : undefined;
        error = error ? function (error) { return observerOrNext.error(error); } : undefined;
        complete = complete ? function () { return observerOrNext.complete(); } : undefined;
    }
    return new SafeSubscriberCtor({
        complete: complete,
        error: error,
        next: next,
    });
}
exports.toSubscriber = toSubscriber;
